# Copyright 2016 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import astroid

from astroid import MANAGER


def __init__(self):
  pass
    

def mutable_record_transform(cls):
  """Transform mutable records usage by updating locals."""
  if not (len(cls.bases) > 0
          and isinstance(cls.bases[0], astroid.Call)
          and cls.bases[0].func.as_string() == 'mutablerecords.Record'):
    return

  try:
    # Add required attributes.
    if len(cls.bases[0].args) >= 2:
      for a in cls.bases[0].args[1].elts:
        cls.locals[a] = [None]

    # Add optional attributes.
    if len(cls.bases[0].args) >= 3:
      for a,b in cls.bases[0].args[2].items:
        cls.locals[a.value] = [None]

  except:
    raise SyntaxError('Invalid mutablerecords syntax')


def register(linter):
  """Register transform with the linter."""
  MANAGER.register_transform(astroid.ClassDef, mutable_record_transform)



# Copyright 2016 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import astroid

from astroid import MANAGER


def __init__(self):
  pass


CONF_NODE = None  # AST node representing the conf module.
CONF_LOCALS = {}
CURRENT_ROOT = None


def transform_declare(node):
  """Transform conf.declare calls by stashing the declared names."""
  global CURRENT_ROOT

  if not (isinstance(node.func, astroid.Attribute)
          and isinstance(node.func.expr, astroid.Name)
          and node.func.expr.name == 'conf'
          and node.func.attrname == 'declare'):
    return

  conf_key_name = None
  if node.args:
    conf_key_name = node.args[0].value
  else:
    for keyword in node.keywords:
      if keyword.arg == 'name':
        # Assume the name is an astroid.Const(str), so it has a str value.
        conf_key_name = keyword.value.value
        break
    assert conf_key_name != None, "Invalid conf.declare() syntax"

  if CONF_NODE:
    # Keep track of the current root, refreshing the locals if it changes.
    if not CURRENT_ROOT or CURRENT_ROOT != node.root():
      CURRENT_ROOT = node.root()
      CONF_NODE.locals = CONF_LOCALS
      CONF_NODE.locals[conf_key_name] = [None]

  else:
    CONF_LOCALS[conf_key_name] = [None]


def transform_conf_module(cls):
  """Transform usages of the conf module by updating locals."""
  global CONF_NODE

  if cls.name == 'openhtf.conf':
    # Put all the attributes in Configuration into the openhtf.conf node.
    cls._locals.update(cls.locals['Configuration'][0].locals)

    # Store reference to this node for future use.
    CONF_NODE = cls
    CONF_LOCALS.update(cls.locals)


def register(linter):
  """Register all transforms with the linter."""
  MANAGER.register_transform(astroid.Call, transform_declare)
  MANAGER.register_transform(astroid.Module, transform_conf_module)



# Copyright 2014 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Setup script for OpenHTF."""

import errno
import glob
import os
import platform
import subprocess
import sys

from distutils.command.build import build
from distutils.command.clean import clean
from distutils.cmd import Command
from distutils.spawn import find_executable
from setuptools import find_packages
from setuptools import setup
from setuptools.command.test import test

HERE = os.path.abspath(os.path.dirname(__file__))

class CleanCommand(clean):
  """Custom logic for the clean command."""

  def run(self):
    clean.run(self)
    targets = [
        './dist',
        './*.egg-info',
        './openhtf/output/proto/*_pb2.py',
        './openhtf/**/*.pyc',
    ]
    os.system('shopt -s globstar; rm -vrf %s' % ' '.join(targets))


class BuildProtoCommand(Command):
  """Custom setup command to build protocol buffers."""
  description = 'Builds the proto files into python files.'
  user_options = [('protoc=', None, 'Path to the protoc compiler.'),
                  ('protodir=', None, 'Path to protobuf install.'),
                  ('indir=', 'i', 'Directory containing input .proto files.'),
                  ('outdir=', 'o', 'Where to output .py files.')]

  def initialize_options(self):
    self.indir = os.getcwd()
    self.outdir = os.getcwd()
    try:
      prefix = subprocess.check_output(
          'pkg-config --variable prefix protobuf'.split()).strip().decode('utf-8')
    except (subprocess.CalledProcessError, OSError):
      if platform.system() == 'Linux':
        # Default to /usr?
        prefix = '/usr'
      elif platform.system() in ['Mac', 'Darwin']:
        # Default to /usr/local for Homebrew
        prefix = '/usr/local'
      else:
        # Handle installing on Windows.
        self.protodir = os.path.join(HERE, 'lib', 'protoc')
        self.protoc = os.path.join(self.protodir, 'bin', 'protoc.exe')
        return

    maybe_protoc = os.path.join(prefix, 'bin', 'protoc')
    if os.path.isfile(maybe_protoc) and os.access(maybe_protoc, os.X_OK):
      self.protoc = maybe_protoc
    else:
      print('Warning: protoc not found at %s' % maybe_protoc)
      print('setup will attempt to run protoc with no prefix.')
      self.protoc = 'protoc'

    self.protodir = os.path.join(prefix, 'include')

  def finalize_options(self):
    pass

  def run(self):
    # Build regular proto files.
    protos = glob.glob(
        os.path.join(self.indir, 'openhtf', 'output', 'proto', '*.proto'))

    if protos:
      print('Attempting to build proto files:\n%s' % '\n'.join(protos))
      cmd = [
          self.protoc,
          '--proto_path', self.indir,
          '--proto_path', self.protodir,
          '--python_out', self.outdir,
      ] + protos
      try:
        subprocess.check_call(cmd)
      except OSError as e:
        if e.errno == errno.ENOENT:
          print('Could not find the protobuf compiler at \'%s\'' % self.protoc)
          if sys.platform.startswith('linux'):
            print('On many Linux systems, this is fixed by installing the '
                  '"protobuf-compiler" and "libprotobuf-dev" packages.')
          elif sys.platform == 'darwin':
            print('On Mac, protobuf is often installed via homebrew.')
        raise
      except subprocess.CalledProcessError:
        print('Could not build proto files.')
        print('This could be due to missing helper files. On many Linux '
              'systems, this is fixed by installing the '
              '"libprotobuf-dev" package.')
        raise
    else:
      print('Found no proto files to build.')


# Make building protos part of building overall.
build.sub_commands.insert(0, ('build_proto', None))


INSTALL_REQUIRES = [
    'colorama>=0.3.9,<1.0',
    'contextlib2>=0.5.1,<1.0',
    'future>=0.16.0',
    'mutablerecords>=0.4.1,<2.0',
    'oauth2client>=1.5.2,<2.0',
    'protobuf>=3.6.0,<4.0',
    'PyYAML>=3.13,<4.0',
    'pyOpenSSL>=17.1.0,<18.0',
    'sockjs-tornado>=1.0.3,<2.0',
    'tornado>=4.3,<5.0',
]
# Not all versions of setuptools support semicolon syntax for specifying
# platform-specific dependencies, so we do it the old school way.
if sys.version_info < (3,4):
  INSTALL_REQUIRES.append('enum34>=1.1.2,<2.0')



class PyTestCommand(test):
  # Derived from
  # https://github.com/chainreactionmfg/cara/blob/master/setup.py
  user_options = [
      ('pytest-args=', None, 'Arguments to pass to py.test'),
      ('pytest-cov=', None, 'Enable coverage. Choose output type: '
       'term, html, xml, annotate, or multiple with comma separation'),
  ]

  def initialize_options(self):
    test.initialize_options(self)
    self.pytest_args = ['test']
    self.pytest_cov = None

  def finalize_options(self):
    test.finalize_options(self)
    self.test_args = []
    self.test_suite = True

  def run_tests(self):
    self.run_command('build_proto')

    import pytest
    cov = []
    if self.pytest_cov is not None:
      outputs = []
      for output in self.pytest_cov.split(','):
        outputs.extend(['--cov-report', output])
      cov = ['--cov', 'openhtf'] + outputs

    sys.argv = [sys.argv[0]]
    print('invoking pytest.main with %s' % (self.pytest_args + cov))
    sys.exit(pytest.main(self.pytest_args + cov))


setup(
    name='spintop',
    version='0.5.2',
    description='A fork of OpenHTF 1.4.2, the open hardware testing framework.',
    author='John Hawley',
    author_email='madsci@google.com',
    maintainer='William Laroche',
    maintainer_email='william.laroche@tackv.ca',
    packages=find_packages(exclude='examples'),
    package_data={'openhtf': ['output/proto/*.proto',
                              'output/web_gui/dist/*',
                              'output/web_gui/dist/css/*',
                              'output/web_gui/dist/js/*',
                              'output/web_gui/dist/img/*',
                              'output/web_gui/*']},
    cmdclass={
        'build_proto': BuildProtoCommand,
        'clean': CleanCommand,
        'test': PyTestCommand,
    },
    install_requires=INSTALL_REQUIRES,
    extras_require={
        'usb_plugs': [
            'libusb1>=1.3.0,<2.0',
            'M2Crypto>=0.22.3,<1.0',
        ],
        'update_units': [
            'xlrd>=1.0.0,<2.0',
        ],
        'serial_collection_plug': [
            'pyserial>=3.3.0,<4.0',
        ],
    },
    setup_requires=[
        'wheel>=0.29.0,<1.0',
    ],
    tests_require=[
        'mock>=2.0.0',
        'pandas>=0.22.0',
        'pytest>=2.9.2',
        'pytest-cov>=2.2.1',
    ],
)



# Copyright 2016 Google Inc. All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import tempfile
import unittest

import mock

import openhtf
from openhtf.core import phase_group
from openhtf.core import test_descriptor
from openhtf.core import test_state
from openhtf.util import conf


@openhtf.measures('test_measurement')
@openhtf.PhaseOptions(name='test_phase')
def test_phase():
  """Test docstring."""
  pass


PHASE_STATE_BASE_TYPE_INITIAL = {
    'name': 'test_phase',
    'codeinfo': {
        'docstring': None,
        'name': '',
        'sourcecode': '',
    },
    # The descriptor id is not static, so this must be updated.
    'descriptor_id': 'MUST_BE_UPDATED',
    'options': None,
    'measurements': {
        'test_measurement': {
            'name': 'test_measurement',
            'outcome': 'UNSET',
        },
    },
    'attachments': {},
}

PHASE_RECORD_BASE_TYPE = copy.deepcopy(PHASE_STATE_BASE_TYPE_INITIAL)
PHASE_RECORD_BASE_TYPE.update({
    'start_time_millis': 0,
    'end_time_millis': None,
    'outcome': None,
    'result': None,
})

TEST_STATE_BASE_TYPE_INITIAL = {
    'status': 'WAITING_FOR_TEST_START',
    'test_record': {
        'station_id': conf.station_id,
        'code_info': None,
        'dut_id': None,
        'start_time_millis': 0,
        'end_time_millis': None,
        'outcome': None,
        'outcome_details': [],
        'metadata': {
            'config': {}
        },
        'phases': [],
        'log_records': [],
    },
    'plugs': {
        'plug_descriptors': {},
        'plug_states': {},
    },
    'running_phase_state': copy.deepcopy(PHASE_STATE_BASE_TYPE_INITIAL),
}


class TestTestApi(unittest.TestCase):

  def setUp(self):
    self.test_descriptor = test_descriptor.TestDescriptor(
        phase_group.PhaseGroup(main=[test_phase]), None, {'config': {}})
    self.test_state = test_state.TestState(self.test_descriptor, 'testing-123',
                                           test_descriptor.TestOptions())
    self.test_record = self.test_state.test_record
    self.running_phase_state = test_state.PhaseState.from_descriptor(
        test_phase, lambda *args: None)
    self.test_state.running_phase_state = self.running_phase_state
    self.test_api = self.test_state.test_api

  def test_get_attachment(self):
    attachment_name = 'attachment.txt'
    input_contents = b'This is some attachment text!'
    mimetype = 'text/plain'
    self.test_api.attach(attachment_name, input_contents, mimetype)

    output_attachment = self.test_api.get_attachment(attachment_name)
    self.assertEqual(input_contents, output_attachment.data)
    self.assertEqual(mimetype, output_attachment.mimetype)

  def test_get_measurement(self):
    measurement_val = [1, 2, 3]
    self.test_api.measurements['test_measurement'] = measurement_val
    measurement = self.test_api.get_measurement('test_measurement')

    self.assertEqual(measurement_val, measurement.value)
    self.assertEqual('test_measurement', measurement.name)

  def test_get_measurement_immutable(self):
    measurement_val = [1, 2, 3]
    self.test_api.measurements['test_measurement'] = measurement_val
    measurement = self.test_api.get_measurement('test_measurement')

    self.assertEqual(measurement_val, measurement.value)
    self.assertEqual('test_measurement', measurement.name)

    measurement.value.append(4)
    self.assertNotEqual(measurement_val, measurement.value)

  def test_infer_mime_type_from_file_name(self):
    f = tempfile.NamedTemporaryFile(delete=False, suffix='.txt')
    f.write(b'Mock text contents.')
    f.flush()
    file_name = f.name
    self.test_api.attach_from_file(file_name, 'attachment')
    f.close()
    attachment = self.test_api.get_attachment('attachment')
    self.assertEqual(attachment.mimetype, 'text/plain')

  def test_infer_mime_type_from_attachment_name(self):
    f = tempfile.NamedTemporaryFile(delete=False)
    f.write(b'Mock text contents.')
    f.flush()
    file_name = f.name
    self.test_api.attach_from_file(file_name, 'attachment.png')
    f.close()
    attachment = self.test_api.get_attachment('attachment.png')
    self.assertEqual(attachment.mimetype, 'image/png')

  def test_phase_state_cache(self):
    basetypes = self.running_phase_state.as_base_types()
    expected_initial_basetypes = copy.deepcopy(PHASE_STATE_BASE_TYPE_INITIAL)
    expected_initial_basetypes['descriptor_id'] = basetypes['descriptor_id']
    self.assertEqual(expected_initial_basetypes, basetypes)
    self.assertFalse(self.running_phase_state._update_measurements)
    self.test_api.measurements.test_measurement = 5
    self.assertEqual({'test_measurement'},
                     self.running_phase_state._update_measurements)
    self.running_phase_state.as_base_types()
    expected_after_basetypes = copy.deepcopy(expected_initial_basetypes)
    expected_after_basetypes['measurements']['test_measurement'].update({
        'outcome': 'PASS',
        'measured_value': 5,
    })
    self.assertEqual(expected_after_basetypes, basetypes)
    self.assertFalse(self.running_phase_state._update_measurements)

  def test_test_state_cache(self):
    basetypes = self.test_state.as_base_types()
    # The descriptor id is not static, so grab it.
    expected_initial_basetypes = copy.deepcopy(TEST_STATE_BASE_TYPE_INITIAL)
    descriptor_id = basetypes['running_phase_state']['descriptor_id']
    expected_initial_basetypes['running_phase_state']['descriptor_id'] = (
        descriptor_id)
    self.assertEqual(expected_initial_basetypes, basetypes)
    self.running_phase_state._finalize_measurements()
    self.test_record.add_phase_record(self.running_phase_state.phase_record)
    self.test_state.running_phase_state = None
    basetypes2 = self.test_state.as_base_types()
    expected_after_phase_record_basetypes = copy.deepcopy(
        PHASE_RECORD_BASE_TYPE)
    expected_after_phase_record_basetypes['descriptor_id'] = descriptor_id
    self.assertEqual(expected_after_phase_record_basetypes,
                     basetypes2['test_record']['phases'][0])
    self.assertIsNone(basetypes2['running_phase_state'])
